from individual import *
import numpy as np

class Group:
    def __init__(self, graphAdj, params, initType):
        self.individualList = {}
        self.params = params

        # 导入节点和邻接关系
        for index, neighbors in graphAdj.items():
            neighbors = list(neighbors)
            self.individualList[index] = Individual(index, neighbors, params)
        self.groupSize = len(self.individualList)
        
        # 建立关系的引用表
        for item in self.individualList.values():
            item.SetupNeighbors(self.individualList)

        # 设置各类人的初始数量
        temp = params['S0'] + params['R0'] + params['In0'] + params['Ip0']
        self.num_R = round(params['R0']/temp*self.groupSize)
        self.num_IP = round(params['Ip0']/temp*self.groupSize)
        self.num_IN = round(params['In0']/temp*self.groupSize)
        self.num_S = self.groupSize - self.num_R - self.num_IP - self.num_IN

        # 各类人的初始数量设置人的初始状态
        if initType == 1: # 随机分布
            randomIndex = list(range(self.groupSize))
            random.shuffle(randomIndex)
            for i in range(self.num_IP):
                self.individualList[randomIndex[0]].state = STATE_IP
                self.individualList[randomIndex[0]].state_last = STATE_IP
                del randomIndex[0]
            for i in range(self.num_IN):
                self.individualList[randomIndex[0]].state = STATE_IN
                self.individualList[randomIndex[0]].state_last = STATE_IN
                del randomIndex[0]
            for i in range(self.num_S):
                self.individualList[randomIndex[0]].state = STATE_S
                self.individualList[randomIndex[0]].state_last = STATE_S
                del randomIndex[0]
            for i in range(self.num_R):
                self.individualList[randomIndex[0]].state = STATE_R
                self.individualList[randomIndex[0]].state_last = STATE_R
                del randomIndex[0]
                
        if initType == 2: # 积极占大节点
            indexAndDegree = [(i, self.individualList[i].degree) for i in range(self.groupSize)]
            indexAndDegree.sort(key=lambda x: x[1], reverse=True)
            for i in range(self.num_IP):
                self.individualList[indexAndDegree[0][0]].state = STATE_IP
                self.individualList[indexAndDegree[0][0]].state_last = STATE_IP
                del indexAndDegree[0]
            random.shuffle(indexAndDegree)
            for i in range(self.num_IN):
                self.individualList[indexAndDegree[0][0]].state = STATE_IN
                self.individualList[indexAndDegree[0][0]].state_last = STATE_IN
                del indexAndDegree[0]
            for i in range(self.num_S):
                self.individualList[indexAndDegree[0][0]].state = STATE_S
                self.individualList[indexAndDegree[0][0]].state_last = STATE_S
                del indexAndDegree[0]
            for i in range(self.num_R):
                self.individualList[indexAndDegree[0][0]].state = STATE_R
                self.individualList[indexAndDegree[0][0]].state_last = STATE_R
                del indexAndDegree[0]

        elif initType == 3: # 消极占大节点
            indexAndDegree = [(i, self.individualList[i].degree) for i in range(self.groupSize)]
            indexAndDegree.sort(key=lambda x: x[1], reverse=True)
            for i in range(self.num_IN):
                self.individualList[indexAndDegree[0][0]].state = STATE_IN
                self.individualList[indexAndDegree[0][0]].state_last = STATE_IN
                del indexAndDegree[0]
            random.shuffle(indexAndDegree)
            for i in range(self.num_IP):
                self.individualList[indexAndDegree[0][0]].state = STATE_IP
                self.individualList[indexAndDegree[0][0]].state_last = STATE_IP
                del indexAndDegree[0]
            for i in range(self.num_S):
                self.individualList[indexAndDegree[0][0]].state = STATE_S
                self.individualList[indexAndDegree[0][0]].state_last = STATE_S
                del indexAndDegree[0]
            for i in range(self.num_R):
                self.individualList[indexAndDegree[0][0]].state = STATE_R
                self.individualList[indexAndDegree[0][0]].state_last = STATE_R
                del indexAndDegree[0]
                
        # 颜色表
        self.colorTable = {
            STATE_S  : 'blue',
            STATE_R  : 'black',
            STATE_IP : 'red',
            STATE_IN : 'green'
            }

        self.GetStatistic()

    def GetStatistic(self): # 统计各类人的数据
        tempList = {
            STATE_S : 0,
            STATE_R : 0,
            STATE_IP : 0,
            STATE_IN : 0,}
        self.colors = [self.colorTable[self.individualList[i].state] for i in range(len(self.individualList))]
        for item in self.individualList.values():
            tempList[item.state] += 1
            
        self.num_S  = tempList[STATE_S]
        self.num_R  = tempList[STATE_R]
        self.num_IP = tempList[STATE_IP]
        self.num_IN = tempList[STATE_IN]


    def Update_Traditional(self, scale = 0.1): # 传统模型
        # 更新人群类型状态
        for item in self.individualList.values():
            item.UpdateState_Traditional(scale)  # 改进的模型

        # 更新人群degree
        for item in self.individualList.values():
            item.UpdateDegree()

        # 更新人群类型状态（前一时刻）
        for item in self.individualList.values():
            item.UpdateLastState()
            

    def Update_Improved(self, scale = 0.1): # 改进模型
        # 更新人群类型状态
        for item in self.individualList.values():
            item.UpdateState_Improved(scale)  # 改进的模型

        # 更新人群degree
        for item in self.individualList.values():
            item.UpdateDegree()

        # 更新人群类型状态（前一时刻）
        for item in self.individualList.values():
            item.UpdateLastState()

    def GetFeature(self):
        # 平均度、最大节点度
        totalDegree = 0
        maxDegree = 0
        minDegree = self.groupSize
        for item in self.individualList.values():
            totalDegree += item.degree
            
            if maxDegree < item.degree:
                maxDegree = item.degree
            if minDegree > item.degree:
                minDegree = item.degree
        self.aveDegree = totalDegree / self.groupSize
        self.maxDegree = maxDegree

        # 度分布
        self.degreeDistribution = np.zeros([3, maxDegree+1], np.float64)
        self.degreeDistribution[2] = np.linspace(minDegree,maxDegree,maxDegree+1, dtype=np.int64)
        for item in self.individualList.values():
            self.degreeDistribution[0,item.degree - maxDegree] += 1
        self.degreeDistribution[1] = self.degreeDistribution[0] / self.groupSize
            

if __name__ == '__main__':
    import networkx as nx
    import matplotlib.pyplot as plt
    params = {
        'α1' : 0.42,
        'α2' : 0.36, 
        'θ1' : 0.04,
        'θ2' : 0.02,  
        'β1' : 0.04,
        'β2' : 0.06,
        'γ'  : 0.22, 
        'ε'  : 0.05,
        'S0'  : 1,
        'In0' : 0,
        'Ip0' : 0,
        'R0'  : 0,
        }

    N = 100 # 网络节点个数N
    K = 5   # K值
    P = 0.3 # 重连概率P
    world = nx.watts_strogatz_graph(N, K, P)
    ps = nx.circular_layout(world)
    #ps = nx.kamada_kawai_layout(world)
    #ps = nx.spiral_layout(world)
    g = Group(world.adj, params)

    g.individualList[0].state = STATE_IP
    
    while True:
        g.Update()
        g.GetStatistic()
        nx.draw(world, ps, with_labels=False, node_size=20, node_color=g.colors, edge_color=[0.7,0.7,0.7])
        plt.show()
        













