import random

STATE_S  = 1
STATE_R  = 2
STATE_IP = 3
STATE_IN = 4

class Individual:
    def __init__(self, index, neighborIndex, params):
        self.index = index
        self.neighborIndex = neighborIndex
        self.params = params

        self.state = STATE_S
        self.state_last = STATE_S

        self.UpdateDegree()

    def SetupNeighbors(self, individualList):
        self.neighbors = []
        for index in self.neighborIndex:
            self.neighbors.append(individualList[index])
        

    def UpdateDegree(self):
        self.degree = len(self.neighborIndex)

    def UpdateState_Improved(self, scale = 0.1): # 更新个体的状态
        neighborsDegree = {
            STATE_S : 0,
            STATE_R : 0,
            STATE_IP : 0,
            STATE_IN : 0,
            }

        # 统计四种类型的相邻节点的总度
        for neighbor in self.neighbors:
            neighborsDegree[neighbor.state_last] += neighbor.degree

        
        A = neighborsDegree[STATE_IP] - neighborsDegree[STATE_IN]

        # 按计算自身的度的方式算B
        B = (neighborsDegree[STATE_S] + neighborsDegree[STATE_R]) / (neighborsDegree[STATE_S] + neighborsDegree[STATE_R] + 0.1) * 1.7

        # 按不计算自身的度的方式算B
        #B = (neighborsDegree[STATE_S] + neighborsDegree[STATE_R]) / (neighborsDegree[STATE_S] + neighborsDegree[STATE_R] + abs(A))


        # 如果自己是S
        if self.state == STATE_S:
            D = B * self.params['γ']
            if A > 0:
                F1 = self.params['α1'] * A / self.degree
                if F1 > 1 - D:
                    F1 = 1 - D
                rnd = random.random() / scale
                if rnd < D:
                    self.state = STATE_R
                elif rnd < D + F1:
                    self.state = STATE_IP
                return
            
            else:
                A = -A
                F2 = self.params['α2'] * A / self.degree
                if F2 > 1 - D:
                    F2 = 1 - D
                rnd = random.random() / scale
                if rnd < D:
                    self.state = STATE_R
                elif rnd < D + F2:
                    self.state = STATE_IN
                return
                

        # 如果自己是R
        elif self.state == STATE_R:
            rnd = random.random() / scale
            if rnd < self.params['ε']:
                self.state = STATE_S

        # 如果自己是IP
        elif self.state == STATE_IP:
            F3 = B * self.params['β1']
            if A < 0:
                A = -A
                F5 = self.params['θ2'] * A / self.degree
                if F5 > 1 - F3:
                    F5 = 1 - F3
            else:
                F5 = 0
                
            rnd = random.random() / scale
            if rnd < F3:
                self.state = STATE_R
            elif rnd < F3 + F5:
                self.state = STATE_IN
            return
                
        # 如果自己是IN
        elif self.state == STATE_IN:
            F4 = B * self.params['β2']
            if A > 0:
                F6 = self.params['θ1'] * A / self.degree
                if F6 > 1 - F4:
                    F6 = 1 - F4
            else:
                F6 = 0
                
            rnd = random.random() / scale
            if rnd < F4:
                self.state = STATE_R
            elif rnd < F4 + F6:
                self.state = STATE_IP
            return


    def UpdateState_Traditional(self, scale = 0.1): # 更新个体的状态
        neighborsDegree = {
            STATE_S : 0,
            STATE_R : 0,
            STATE_IP : 0,
            STATE_IN : 0,
            }

        # 统计四种类型的相邻节点的总度
        for neighbor in self.neighbors:
            neighborsDegree[neighbor.state_last] += neighbor.degree
        neighborsDegree[self.state_last] += self.degree
        totalDegree = neighborsDegree[STATE_S] + neighborsDegree[STATE_R] + neighborsDegree[STATE_IP] + neighborsDegree[STATE_IN]
        
        # 如果自己是S
        if self.state == STATE_S:
            P_IP = neighborsDegree[STATE_IP] / totalDegree * self.params['α1'] * 1
            P_IN = neighborsDegree[STATE_IN] / totalDegree * self.params['α2'] * 1
            P_R  = self.params['γ']  * 1
            rnd = random.random() / scale
            if rnd < P_IP:
                self.state = STATE_IP
            elif rnd < P_IP + P_IN:
                self.state = STATE_IN
            elif rnd < P_IP + P_IN + P_R:
                self.state = STATE_R
            return

        # 如果自己是R
        elif self.state == STATE_R:
            P_R  = self.params['ε']  * 1
            rnd = random.random() / scale
            if rnd < P_R:
                self.state = STATE_S
            return

        # 如果自己是IP
        elif self.state == STATE_IP:
            P_R  = self.params['β1'] * 1
            #P_IN = neighborsDegree[STATE_IN] / totalDegree * self.params['θ2'] * 1
            P_IN = self.params['θ2'] * 1
            rnd = random.random() / scale
            if rnd < P_R:
                self.state = STATE_R
            elif rnd < P_R + P_IN:
                self.state = STATE_IN
            return
            
        # 如果自己是IN
        elif self.state == STATE_IN:
            P_R  = self.params['β2'] * 1
            #P_IP = neighborsDegree[STATE_IP] / totalDegree * self.params['θ1'] * 1
            P_IP = self.params['θ1'] * 1
            rnd = random.random() / scale
            if rnd < P_R:
                self.state = STATE_R
            elif rnd < P_R + P_IP:
                self.state = STATE_IP
            return

    def UpdateLastState(self):
        self.state_last = self.state
