import networkx as nx
import matplotlib.pyplot as plt
from group import Group
from individual import *
import numpy as np

from my_csv import *

# 设置参数
params = {
    'α1' : 0.40,
    'α2' : 0.30,
    'θ1' : 0.05,
    'θ2' : 0.02,  
    'β1' : 0.06,
    'β2' : 0.06,
    'γ'  : 0.1,
    'ε'  : 0.05,
    'S0'  : 0.96,
    'In0' : 0.02,
    'Ip0' : 0.02,
    'R0'  : 0.0,
    }

# 网络模型
networkType = 3 # BA网络
#networkType = 2 # 小世界网络
#networkType = 3 # 微博网络

# 节点行为模型
#moduleType = 1 # 传统模型
moduleType = 2 # 改进模型

# 初始状态分布
initType = 1 # 随机分布
#initType = 2 # 积极占大节点
#initType = 3 # 消极占大节点

if networkType == 1:
    # 创建 BA 网络模型
    world = nx.barabasi_albert_graph(10000,4) #generate BA network
    ps = nx.spring_layout(world) #set layout

if networkType == 2:
    # 创建小世界网络模型
    N = 10000 #int(input('网络节点个数N: '))
    K = 8   #int(input('K值: '))
    P = 0.1  #float(input('重连概率P：'))
    world = nx.watts_strogatz_graph(N, K, P)
    ps = nx.circular_layout(world)

if networkType == 3:
    # 创建微博网络模型
    relationship = readcsv('关系3.csv')
    world = nx.Graph()
    maxUserNum = 3000 # 限制最多只加载 maxUserNum 个节点，微博网络数据共528390节点，
    for i in range(maxUserNum):
        world.add_node(i)
    for id_1, id_2 in relationship:
        id_1 = int(id_1)
        id_2 = int(id_2)
        if id_1 < maxUserNum and id_2 < maxUserNum:
            world.add_edge(id_1, id_2)

# 建立群体和仿真
g = Group(world.adj, params, initType)

# 计算模型参数
print('节点数：', world.number_of_nodes())

print('边数：', world.number_of_edges())

temp = nx.clustering(world)
temp = np.array(list(temp.values()))
print('聚类系数：', temp.mean())

print('度相关系数：', nx.degree_pearson_correlation_coefficient(world))

g.GetFeature()
print('平均度：', g.aveDegree)

print('最大节点度：', g.maxDegree)

# 度分布图
fig, ax = plt.subplots(1,1)
ax.plot(g.degreeDistribution[2], g.degreeDistribution[1], 'r.')
ax.set_xscale('log')
ax.set_yscale('log')
#plt.ion()
#plt.show() # 只看度分布图


steps = 40   # 轮数
scale = 1    # 步长
n = 1     # 实验次数

counts = int(steps/scale)
totalData = np.zeros([5, counts])

for i in range(n):
    S  = []
    R  = []
    IP = []
    IN = []
    ii = []
    counter = 0
    g = Group(world.adj, params, initType)
    while counter < counts:
        #print(g.num_S, g.num_R, g.num_IP, g.num_IN)
        S.append(g.num_S)
        R.append(g.num_R)
        IP.append(g.num_IP)
        IN.append(g.num_IN)
        ii.append(counter * scale)

        if moduleType == 1:
            g.Update_Traditional(scale)  # 使用传统模型
        elif moduleType == 2:
            g.Update_Improved(scale)     # 使用改进模型

        
        g.GetStatistic()
        #nx.draw(world, ps, with_labels=False, node_size=30, node_color=g.colors, edge_color=[0.7,0.7,0.7])
        #plt.show()
        counter += 1
    totalData += np.array([S, R, IP, IN, ii])

fig, ax = plt.subplots(1,1)

aveData = totalData / n
ave_S  = aveData[0]
ave_R  = aveData[1]
ave_IP = aveData[2]
ave_IN = aveData[3]
ave_ii = aveData[4]
ax.plot(ave_ii, ave_R,  'b--',  label='IN(t)realcase')
#ax.plot(ave_ii, ave_R,  'r-.',  label='IN(t)simulationdata')
#ax.plot(ave_ii, ave_S,  '--',  label='S(t)')
#ax.plot(ave_ii, ave_R,  'black', label='R(t)')
#ax.plot(ave_ii, ave_IP, '-.',   label='IP(t)')
#ax.plot(ave_ii, ave_IN, 'r:', label='IN(t)')
ax.legend(fontsize=10) 
plt.show()
