from scipy.integrate import odeint  # 导入 scipy.integrate 模块
import numpy as np  # 导入 numpy包
import matplotlib.pyplot as plt  # 导入 matplotlib包

def dySt(y, t, α1,α2,γ,ε,θ1,θ2,β1,β2):
    S,Ip,In,R = y
    ds_dt = -α1*S*Ip-α2*S*In-γ*S+ε*R
    dIp_dt = α1*S*Ip+θ1*In-θ2*Ip-β1*Ip
    dIn_dt = α2*S*In+θ2*Ip-θ1*In-β2*In
    dR_dt = β1*Ip+β2*In+γ*S-ε*R
    return np.array([ds_dt,dIp_dt,dIn_dt,dR_dt])

# 设置模型参数
α1 = 0.49
α2 = 0.41
θ1 = 0.05
θ2 = 0.02
β1 = 0.06
β2 = 0.04
γ = 0.1
ε = 0.03
S0 = 0.96
In0 = 0.02
Ip0 = 0.02
R0 = 0

tEnd = 100
t = np.arange(0.0,tEnd,1)
yS = odeint(dySt, (S0,Ip0,In0,R0), t, args=(α1,α2,γ,ε,θ1,θ2,β1,β2))
#yIp = odeint(dyIpt, (S0,In0,Ip0,R0), t, args=(α1,θ1,θ2,β1))
#yIn = odeint(dyInt, (S0,In0,Ip0,R0), t, args=(α2,θ1,θ2,β2))
#yR = odeint(dyRt, (S0,In0,Ip0,R0), t, args=(β1,β2,γ,ε))
plt.figure(figsize=(13, 10))
plt.xlabel('t', fontsize=20)
plt.axis([0, tEnd, 0, 1])
plt.plot(t, yS[:,0], 'b.-', label='S(t)')
plt.plot(t, yS[:,1], 'gx:',  label='Ip(t)')
plt.plot(t, yS[:,2], 'r:',  label='In(t)')
plt.plot(t, yS[:,3], 'orange',  label='R(t)')
plt.legend(fontsize=10)
plt.savefig("图.png")
plt.show()
'''
def dySt(y,t,α,ε,β):
    S,I,R = y
    ds_dt = -α*S*I+ε*R
    dI_dt = α*S*I-β*I
    dR_dt = β*I-ε*R
    return np.array([ds_dt,dI_dt,dR_dt])

# 设置模型参数
α = 0.4
β = 0.08
ε = 0.03
S0 = 0.96
I0 = 0.04
R0 = 0.0

tEnd = 100
t = np.arange(0.0,tEnd,1)
yS = odeint(dySt, (S0,I0,R0), t, args=(α,ε,β))
#yI = odeint(dyIt, (S0,I0,R0), t, args=(α,β))
#yR = odeint(dyRt, (S0,I0,R0), t, args=(β,ε))
plt.figure(figsize=(13, 10))
plt.xlabel('t', fontsize=20)
plt.axis([0, tEnd, 0, 1])
plt.plot(t, yS[:,0], 'b.-', label='S(t)')
plt.plot(t, yS[:,1], 'gx:',  label='I(t)')
plt.plot(t, yS[:,2], 'orange',  label='R(t)')
plt.legend(fontsize=10)
plt.savefig("图.png")
plt.show()
